@@ -32,18 +32,18 @@ class OpenApiArt(object):
         api_files,
         python_module_name=None,
         protobuf_package_name=None,
-        protobuf_file_name=None,
+        go_sdk_package_dir=None,
+        go_sdk_package_name=None,
         output_dir=None,
-        extension_prefix = None
+        extension_prefix=None,
     ):
         self._python_module_name = python_module_name
-        self._protobuf_file_name = protobuf_file_name
         self._protobuf_package_name = protobuf_package_name
+        self._go_sdk_package_dir = go_sdk_package_dir
+        self._go_sdk_package_name = go_sdk_package_name
         self._extension_prefix = extension_prefix
-        if output_dir is None:
-            output_dir = os.path.join(os.getcwd(), ".output")
-        self._relative_output_dir = output_dir
         self._output_dir = os.path.abspath(output_dir)
+        print("Artifact output directory: {output_dir}".format(output_dir=self._output_dir))
         shutil.rmtree(self._output_dir, ignore_errors=True)
         self._api_files = api_files
         self._bundle()
@@ -54,27 +54,14 @@ class OpenApiArt(object):
 
     def _get_license(self):
         try:
-            self._license = "License: {}".format(
-                self._bundler._content["info"]["license"]["url"]
-            )
-            return
-            # currently license URL returns an HTML and not solely license text
-            # hence skipping this part unless we come across a better way to
-            # parse licenses
-            response = requests.request("GET", self._bundler._content["info"]["license"]["url"])
-            if response.ok:
-                self._license = response.text
-            else:
-                raise Exception(response.text)
-        except Exception as e:
-            self._license = "OpenAPI info.license.url error [{}]".format(e)
+            license_name = self._bundler._content["info"]["license"]["name"]
+            self._license = "License: {}".format(license_name)
+        except:
+            raise Exception("The /info/license/name is a REQUIRED property and must be present in the schema.")
 
     def _get_info(self):
         try:
-            self._info = "{} {}".format(
-                self._bundler._content["info"]["title"],
-                self._bundler._content["info"]["version"]
-            )
+            self._info = "{} {}".format(self._bundler._content["info"]["title"], self._bundler._content["info"]["version"])
         except Exception as e:
             self._info = "OpenAPI info error [{}]".format(e)
 
@@ -98,57 +85,87 @@ class OpenApiArt(object):
                 "--output",
                 os.path.join(self._output_dir, "openapi.html"),
             ]
-            process = subprocess.Popen(process_args, shell=True)
-            process.wait()
+            subprocess.check_call(process_args, shell=True)
         except Exception as e:
             print("Bypassed creation of static documentation [missing redoc-cli]: {}".format(e))
 
     def _generate(self):
-        # this writes python ux module
+        # this generates the python ux module
         if self._python_module_name is not None:
             module = importlib.import_module("openapiart.generator")
-            python = getattr(module, "Generator")(
+            python_ux = getattr(module, "Generator")(
                 self._bundler.openapi_filepath,
                 self._python_module_name,
                 output_dir=self._output_dir,
-                extension_prefix = self._extension_prefix
+                extension_prefix=self._extension_prefix,
             )
-            python.generate()
+            python_ux.generate()
 
-        # this generates protobuf definitions
-        try:
+        # this generates protobuf definitions into the output_dir
+        if self._protobuf_package_name and self._go_sdk_package_dir:
             module = importlib.import_module("openapiart.openapiartprotobuf")
             protobuf = getattr(module, "OpenApiArtProtobuf")(
                 **{
                     "info": self._info,
                     "license": self._license,
-                    "python_module_name": self._python_module_name,
-                    "protobuf_file_name": self._protobuf_file_name,
                     "protobuf_package_name": self._protobuf_package_name,
+                    "go_sdk_package_dir": self._go_sdk_package_dir,
                     "output_dir": self._output_dir,
                 }
             )
             protobuf.generate(self._openapi)
-        except Exception as e:
-            print("Bypassed creation of protobuf file: {}".format(e))
 
+        # this generates the python stubs into the output dir/python module
         try:
-            grpc_dir = os.path.normpath(os.path.join(self._output_dir, self._python_module_name))
-            proto_path = os.path.normpath(os.path.join("./"))
+            python_sdk_dir = os.path.normpath(os.path.join(self._output_dir, self._python_module_name))
             process_args = [
                 sys.executable,
                 "-m",
                 "grpc_tools.protoc",
-                "--python_out={}".format(grpc_dir),
-                "--grpc_python_out={}".format(grpc_dir),
+                "--python_out={}".format(python_sdk_dir),
+                "--grpc_python_out={}".format(python_sdk_dir),
                 "--proto_path={}".format(self._output_dir),
-                "{}.proto".format(self._protobuf_file_name),
+                "{}.proto".format(self._protobuf_package_name),
             ]
-            print("grpc_tools.protoc args: {}".format(" ".join(process_args)))
-            process = subprocess.Popen(process_args, shell=False)
-            process.wait()
+            print("Generating python grpc stubs: {}".format(" ".join(process_args)))
+            subprocess.check_call(process_args, shell=False)
         except Exception as e:
-            print("Bypassed creation of python grpc files: {}".format(e))
+            print("Bypassed creation of python stubs: {}".format(e))
+
+        # this generates the go stubs
+        if self._go_sdk_package_dir and self._protobuf_package_name:
+            go_sdk_output_dir = os.path.normpath(os.path.join(self._output_dir, "..", os.path.split(self._go_sdk_package_dir)[-1]))
+            go_protobuffer_out_dir = os.path.normpath(os.path.join(go_sdk_output_dir, self._protobuf_package_name))
+            if not os.path.exists(go_protobuffer_out_dir):
+                os.makedirs(go_protobuffer_out_dir)
+            process_args = [
+                "protoc",
+                "--go_opt=paths=source_relative",
+                "--go-grpc_opt=paths=source_relative",
+                "--go_out={}".format(go_protobuffer_out_dir),
+                "--go-grpc_out={}".format(go_protobuffer_out_dir),
+                "--proto_path={}".format(self._output_dir),
+                "--experimental_allow_proto3_optional",
+                "{}.proto".format(self._protobuf_package_name),
+            ]
+            print("Generating go stubs: {}".format(" ".join(process_args)))
+            subprocess.check_call(process_args, shell=False)
+
+        # this generates the go ux module
+        if self._protobuf_package_name and self._go_sdk_package_dir:
+            module = importlib.import_module("openapiart.openapiartgo")
+            go_ux = getattr(module, "OpenApiArtGo")(
+                **{
+                    "info": self._info,
+                    "license": self._license,
+                    "protobuf_package_name": self._protobuf_package_name,
+                    "go_sdk_package_dir": self._go_sdk_package_dir,
+                    "go_sdk_package_name": self._go_sdk_package_name,
+                    "output_dir": self._output_dir,
+                }
+            )
+            print("Generating go ux sdk: {}".format(" ".join(process_args)))
+            go_ux.generate(self._openapi)
 
     @property
     def output_dir(self):
